#!/usr/bin/env python
''' game mode section '''
import random
import pygame
from pygame.locals import *
from themap import *
from player import *
from enemy import *
from headsupdisplay import *
from enemy_slime import *
from enemy_magma import *
from others import *
from keys import *


class game():
    def __init__(self, mapsize=(100,100)):
        ''' Initializes and starts the game mode '''
        random.seed()
        self.inInit = True          # we are initializing things
        self.tset = tileset()       # Tileset
        self.gmap = themap(self.tset)    # Game Map
        
        self.nextEnemyID = 0        # Next ID # for enemies
        
        self.targetedID = -1        # Targeted Monster ID
        self.nextOtherID = 0        # Next ID # for others
        self.otherlist = []         # list of other objects
        self.keylist = []           # list of keys
        self.nextKeyID = 0          # Next ID # for keys
        self.score = 0              # score variable
        self.totalKeys = 10         # Total number of keys
        
        self.gmap.genmap(mapsize)   # Generate random game map
        self.player = player(self.tset.getXY([63,8]),self)   # create the player
        self.hud = hud(0,self.player.life,self.player.ammo,0,self.totalKeys)# creates the HUD
        self.placeEnemies(200)      # Place enemies on the map
        self.placeKeys(self.totalKeys) # Of keys to place
        
        self.inInit = False         # done intializing things
        self.gameOver = False       # is the Game Over
    
    def render(self, screen):
        ''' renders the screen for game mode
            screen = main display surface
        '''
        self.gmap.render(screen)    # Render background
        
        for o in self.otherlist:    # Render others
            if(self.gmap.isInWindow(o.loc)):
                ''' test if inside view window '''
                o.render(screen, self.gmap.getPixelXY(o.loc))
        
        for k in self.keylist:  # Render keys
            if(self.gmap.isInWindow(k.loc)):
                ''' test if inside view window '''
                k.render(screen, self.gmap.getPixelXY(k.loc))
        
        for e in self.enemylist:    # Render Enemies
            if(self.gmap.isInWindow(e.loc)):
                ''' test if inside view window '''
                e.render(screen, self.gmap.getPixelXY(e.loc))
                if(e.idno == self.targetedID):
                    screen.blit(self.tset.getXY([10,0]), self.gmap.getPixelXY(e.loc))
                
        self.player.render(screen, self.gmap.px, self.gmap.py)  # Render Player
        
        self.hud.render(screen)     # Render HUD
    
    ''' **** EVENT HANDLERS **** '''
    
    def keyhandler(self, event):
        ''' handles keypress events, called by main
            event = PyGame KEYDOWN event
        '''
        self.updatePlayerMap()
        self.player.keyhandler(event)
        if(event.key == K_SPACE and self.targetedID != -1):
            #print self.targetedID
            #temp = None
            for e in self.enemylist:
                if e.idno == self.targetedID:
                    if(self.player.ammo > 0):
                        e.splat()
                        pygame.event.post(pygame.event.Event(USEREVENT,
                                             uetype="ammo",
                                             value=-1))
                   # temp = e
            #self.enemylist.remove(temp)
            self.targetedID = -1
     
    def splatHandler(self, event):
        ''' Handles splatEvents '''
        if event.splattype == 1:
            for y in xrange(-1,2):
                for x in xrange(-1,2):
                    if(self.gmap.getType([event.loc[0]+x, event.loc[1]+y]) == 1):
                        self.otherlist.append(others([event.loc[0]+x, event.loc[1]+y],
                            event.pic,
                            self.tset,
                            self.nextOtherID))
                        self.nextOtherID += 1
        temp = None
        for e in self.enemylist:
            if e.idno == event.idno:
                temp = e
                break
        self.enemylist.remove(temp)
        self.damageCheck()

    def mousehandler(self, event):
        ''' handles mouse events, called by main
            event = PyGame MOUSEBUTTONDOWN event
        '''
        if event.button == 1:
            ''' only deal with left button clicks '''
            xy = self.gmap.getXYfromPixel(event.pos)
            self.targetXY(xy)
    
    def ppos(self, event):
        ''' a player position event has occurred
            event = PyGame USEREVENT with "ppos" key
        '''
        #self.damageCheck() # If player is on splatter damage them
        
        self.gmap.ppos(event.x, event.y)    # Update player position
        
        self.keyCheck() # If player is on a key pick it up
        
        self.damageCheck() # If player is on splatter damage them
        
        if self.inInit == True:    # Trigger updates of enemy positions
            return
        
        for e in self.enemylist:
            e = self.updateEnemyMap(e)
        
        for e in self.enemylist:
            e = self.updateEnemyMap(e)
            e.update()      # run the enemy's update routine
            e = self.updateEnemyMap(e)  # make sure everything is accurate
    
    def scoreHandler(self, event):
        ''' Handles score events '''
        self.score += event.value
        self.hud.updateScore(self.score)
    
    def ammoHandler(self, event):
        ''' Handles ammo events '''
        self.player.ammo += event.value
        self.hud.updateAmmo(self.player.ammo)
    
    def keyEventHandler(self, event):
        ''' Handles key events '''
        self.player.keys += 1
        self.hud.updateKeys(self.player.keys)
        temp=None
        for k in self.keylist:
            if k.idno == event.idno:
                temp = k
        self.keylist.remove(temp)
        pygame.event.post(pygame.event.Event(USEREVENT,
                                             uetype="life",
                                             value=3))
    
    def lifeHandler(self, event):
        ''' Handles life events '''
        self.player.life += event.value
        self.hud.updateLife(self.player.life)
        if self.player.life <= 0:
            pygame.event.post(pygame.event.Event(USEREVENT,
                                             uetype="gameover",
                                             score=self.score))
    
    ''' **** GENERAL SETUP **** '''
    
    def placeEnemies(self, qty):
        ''' places enemies on the map, used after generating map
            qty = number of enemies to generate and place
        '''
        random.seed()
        self.enemylist = []     # Holds all enemies on map
        for e in xrange(0,qty):
            ''' generate and add enemies to enemylist '''
            x = 0
            y = 0
            while(self.gmap.getType([x,y]) != 1):
                ''' so we don't place on a feature '''
                x = random.randint(1,98)
                y = random.randint(1,98)
            mtype = random.randint(1,2)
            if mtype == 1:
                self.enemylist.append(enemy_slime(self.tset, self.nextEnemyID, loc=[x,y]))
            else:
                self.enemylist.append(enemy_magma(self.tset, self.nextEnemyID, loc=[x,y]))
            self.nextEnemyID += 1
        
        for e in self.enemylist:
            e = self.updateEnemyMap(e)
            #print e.mapstate
    
    def placeKeys(self, qty):
        ''' places keys on the map, used after generating map
            qty = number of keys to place
        '''
        random.seed()
        self.keylist = []
        for k in xrange(0,qty):
            x = 0
            y = 0
            while(self.gmap.getType([x,y]) != 1):
                ''' only place on stone floor '''
                x = random.randint(1,98)
                y = random.randint(1,98)
            self.keylist.append(key(self.tset, [54,45], [x,y], self.nextKeyID))
            self.nextKeyID += 1
    
    ''' **** GENERAL GAMEPLAY **** '''
    
    def updateEnemyMap(self, badguy):
        ''' updates enemy map for navigation etc... '''
        a = 0   # a = enemy.mapstate[X][Y] -- A = X
        b = 0   # b = enemy.mapstate[X][Y] -- B = Y
        otherfound = False
        for y in xrange(badguy.loc[1]-1,badguy.loc[1]+2):
            for x in xrange(badguy.loc[0]-1, badguy.loc[0]+2):
                if x<0 or y<0 or x>self.gmap.mapsize[0] or y>self.gmap.mapsize[1]:
                    badguy.mapstate[a][b] = -1
                    badguy.otherstate[a][b] = -1
                else:
                    badguy.mapstate[a][b] = self.gmap.getType([x,y])
                    for e in self.enemylist:
                        if(e.loc[0] == x and e.loc[1] == y):
                            badguy.otherstate[a][b] = e.ENEMYTYPE
                            otherfound = True
                        break   # exit for loop early
                    if otherfound == False:
                        if self.player.x == x and self.player.y == y:
                            badguy.otherstate[a][b] = 999
                        else:
                            badguy.otherstate[a][b] = 0
                otherfound = False  # reset for next iteration
                a += 1  # increment a
            b += 1  # increment b
            a = 0   # reset a
        return badguy
    
    def updatePlayerMap(self):
        ''' updates player map for collision checking '''
        a = 0   # a = enemy.mapstate[X][Y] -- A = X
        b = 0   # b = enemy.mapstate[X][Y] -- B = Y
        for y in xrange(self.player.y-1,self.player.y+2):
            for x in xrange(self.player.x-1, self.player.x+2):
                otherfound = False
                for e in self.enemylist:
                    if(e.loc[0] == x and e.loc[1] == y):
                        print self.player.x, self.player.y, e.loc[0], e.loc[1]
                        self.player.otherstate[a][b] = e.ENEMYTYPE
                        otherfound = True
                        break   # exit for loop early
                if otherfound == False:
                    self.player.otherstate[a][b] = 0
                a += 1  # increment a
            b += 1  # increment b
            a = 0   # reset a
    
    def targetXY(self, pos):
        ''' returns ID # of enemy being targeted
            pos = [x,y] map coordinate of targeted square
        '''
        for e in self.enemylist:
            if e.loc[0] == pos[0] and e.loc[1] == pos[1]:
                if self.targetedID == e.idno:
                    self.targetedID = -1
                else:
                    self.targetedID = e.idno
                break
    
    def keyCheck(self):
        ''' checks if player is on a key '''
        for k in self.keylist:
            if k.loc == self.player.loc:
                pygame.event.post(pygame.event.Event(USEREVENT,
                                             uetype="key",
                                             idno = k.idno))
    
    def damageCheck(self):
        ''' checks if player is on splatter '''
        for o in self.otherlist:
            if o.loc == self.player.loc:
                pygame.event.post(pygame.event.Event(USEREVENT,
                                                     uetype="life",
                                                     value=-1))
                return
    
