#!/usr/bin/env python

''' IMPORTS '''
import pygame                   # Pygame Library
import sys                      # System Library
from pygame.locals import*      # local variables for Pygame
''' classes for this game '''
from menu import *              # Menu Class
from game import *              # Game Class
from highscores import *        # High Scores Class

''' BASIC INITIALIZATION STUFF '''
pygame.init()           # Init Pygame
fps = 30                # Game FPS
scrn_size = (800,600)   # screen size (x,y)

screen = pygame.display.set_mode(scrn_size)
pygame.display.set_caption("Get Out")
clock = pygame.time.Clock()
scores = highscores()

''' Some Definitions '''
def menuQuit():
    pygame.event.post(pygame.event.Event(QUIT,{}))

def menuScores():
    pygame.event.post(pygame.event.Event(USEREVENT,uetype="scores"))

def menuStart():
    pygame.event.post(pygame.event.Event(USEREVENT,uetype="start"))

def gameover(score):
    ''' updates score to the scores table '''
    scores.updateScores(score)
    

''' Menu Initialization Stuff '''
inMenu = True       # Are we in the menu system?
inGame = False      # Are we in the game?
inScores = False    # Are we in the scores screen
menutitle = "menu_item_01"
menuitems = ["menu_item_02",
             "menu_item_04",
             "menu_item_03"]
menuactions = [menuStart,menuScores,menuQuit]

mainmenu = menu(menutitle, menuitems, menuactions)

''' Game Loop '''
while True:
    ''' Handle Events '''
    for event in pygame.event.get():
        if event.type == QUIT:
            pygame.quit()
            sys.exit()
            
        if event.type == KEYDOWN:
            if inMenu:
                ''' if inside a menu '''
                mainmenu.keyHandler(event)
            if inGame:
                ''' if inside the game '''
                thegame.keyhandler(event)
            if inScores:
                scores.keyHandler(event)
        
        if event.type == MOUSEBUTTONDOWN:
            if inGame:
                ''' if inside the game '''
                thegame.mousehandler(event)
                
        if event.type == USEREVENT:
            if event.uetype=="start":
                inMenu = False
                inGame = True
                pygame.mixer.music.stop()
                thegame = game()
            elif event.uetype=="scores":
                inMenu = False
                inScores = True
                thegame = None
            elif event.uetype=="ppos":
                thegame.ppos(event)
            elif event.uetype=="exitgame":
                inGame = False
                inMenu = True
                inScores = False
            elif event.uetype=="splat":
                thegame.splatHandler(event)
            elif event.uetype=="score":
                thegame.scoreHandler(event)
            elif event.uetype=="ammo":
                thegame.ammoHandler(event)
            elif event.uetype=="key":
                thegame.keyEventHandler(event)
            elif event.uetype=="life":
                thegame.lifeHandler(event)
            elif event.uetype=="gameover":
                ''' game over event '''
                inScores = True
                inGame = False
                gameover(event.score)
    
    ''' Sound Rendering '''
    if inMenu:
        ''' in menu play menu music '''
        if pygame.mixer.music.get_busy() == False:
            pygame.mixer.music.load("music_and_credits/song_01.wav")
            pygame.mixer.music.play(-1)
    
    ''' Screen Rendering '''
    
    if inMenu:
        ''' if inside menu '''
        screen.fill((0,0,0))
        screen.blit(mainmenu.renderMenu(), (200,200))
    if inGame:
        ''' if inside game '''
        thegame.render(screen)
    if inScores:
        ''' in scores screen '''
        scores.render(screen)
        
    pygame.display.update()
    
    ''' Frame Rate Related '''
    clock.tick(fps)
