#!/usr/bin/env python
'''
a player sprite class
'''
import pygame
from pygame.locals import *
from sprite import *

class player(sprite):
    '''
    a player sprite
    pic = pygame surface
    life = player life
    speed = player speed
    '''
    def __init__(self,pic,game,x=12,y=10,life=10,speed=2):
        sprite.__init__(self,x,y,pic)
        self.pic = pic
        self.life = life
        self.speed = speed
        self.otherstate = [
            [0,0,0],
            [0,0,0],
            [0,0,0]
        ]
        self.keys = 0   # number of keys found
        self.ammo = 10  # amount of ammo
        
        while(game.gmap.getType([x,y])!= 1):
            ''' find a normal spot on the map '''
            x += 1
            y += 1
        self.x = x
        self.y = y
        self.loc = [self.x, self.y]
        ''' post an event to notify others of player position '''
        pygame.event.post(pygame.event.Event(USEREVENT,
                                             uetype="ppos",
                                             x=self.x,
                                             y=self.y))
        self.game = game
    
    def render(self, screen, px, py):
        ''' render to the screen '''
        screen.blit(self.pic, (px*32,py*32))
    
    def keyhandler(self, event):
        ''' handle keypress events '''
        if event.type == KEYDOWN:
            if event.key == K_UP or event.key == K_w:
                self.move("up")
            elif event.key == K_DOWN or event.key == K_s:
                self.move("down")
            elif event.key == K_RIGHT or event.key == K_d:
                self.move("right")
            elif event.key == K_LEFT or event.key == K_a:
                self.move("left")
            elif event.key == K_ESCAPE:
                pygame.event.post(pygame.event.Event(USEREVENT,
                                                     uetype="exitgame"))
    
    def move(self, direction):
        ''' move the player '''
        #self.game.updatePlayerMap()
        if direction == "up":
            if(self.game.gmap.getType([self.x, self.y-1]) == 1
               and self.otherstate[1][0] == 0):
                self.y -= 1
        elif direction == "down":
            if(self.game.gmap.getType([self.x, self.y+1]) == 1
               and self.otherstate[1][2] == 0):
                self.y += 1
        elif direction == "right":
            if(self.game.gmap.getType([self.x+1, self.y]) == 1
               and self.otherstate[2][1] == 0):
                self.x += 1
        elif direction == "left":
            if(self.game.gmap.getType([self.x-1, self.y]) == 1
               and self.otherstate[0][1] == 0):
                self.x -= 1
        
        self.loc = [self.x, self.y]
        
        ''' post an event to notify others of player position '''
        pygame.event.post(pygame.event.Event(USEREVENT,
                                             uetype="ppos",
                                             x=self.x,
                                             y=self.y))